﻿// Gallery Wrap Creator - v1.0   Dec 6 2008
//This Photoshop script is released as freeware. Do what you will, as long as you don't charge for it and as long as this notice remains intact...
//Copyright ©2008 Chris Ogden <www.InTheMomentImages.com>  
//email: PhotoshopScripts@InfoMatters.com (note: I travel a great deal so it can literally take me weeks to catch up on emails sometimes, esp. after being in Antarctica, Iceland, etc. for a month at a time)
//
// You can run this by double clicking the script, or you may access it from the File-> Scripts menu, if you put this file into the "Scripts "folder (inside "Presets", which is inside your Photoshop folder)
//
//Printing your photo as a "Gallery Wrapped Canvas" requires your image to have borders that can be wrapped around an internal frame of “stretcher bars” and then attached to the frame's back. When hung on a wall, the resulting box's sides protrude from the wall, showing its printed edges. The resulting artwork has depth and shows the viewers “art“ from all angles.
//One excellent way of creating the borders is to extend your image by copying strips from the edges, flipping the copies, expanding the canvas, and moving the newly created edges outward to abut the original photo's sides. This method creates a photo with "mirrored" edges that can then be wrapped.
// This script automates that process, creating a separate adjustment layer with the mirrored borders, one with crop marks, and guides showing the original image.
// Asking the user how wide they'd would like the border (eg, 2" is common for a 1.75" stretcher bars):
//
//PS. A very detailed tutorial about turning your Pixels to Print™ on canvas may be found in our eBook, available at http://InTheMomentImages.com/canvas

//CLO To Do: if there's a background layer, the current scrip fills the new space when resizing the canvas with the foreground color. It'd be nice if I dup'd the background before resizing so the orig image stands on its own.
//CLO To Do: reduce system load and script runtime by flipping the wrapped edges in a new, temp document (to save flipping whole canvas 4 times). Although on my machine a 600MB layered image only takes <30s to run, on older G4 it takes 8minutes.
//CLO TO DO: maybe give user option to restore their selected layer? or leave new layer selected? activeDocument = initialLayer; //restore user's selected layer
// **CLO TO DO: check compatibility on old PS versions (update version check appropriately)

// enable double-clicking from Mac Finder or Windows Explorer
// this command only works in Photoshop CS2 and higher
#target photoshop

var forceQuit = false; //apparently there's no way to quit / exit / stop a script in the main body, so use a variable (could put main into a function but then have to go back and make everything follow proper OO steps) 

var psVersion = getAppVersion(); // alert(psVersion);
if (getAppVersion() < 9) {	
	alert('Hmmm, looks like you\'re running an older version of Photoshop with which we haven\'t yet tested this script fully. \r\rYou can try to run it to see what happens. As long as you\'ve saved your image beforehand, the worst that should happen is that if the script fails part way through, you\'ll want to close your document (without saving) and re-open it.');
	//forceQuit = true; // quit
}	


// ensure that there is at least one document open
if (!documents.length) {
	alert('There are no documents open (so we can\'t run the script). \r\rPlease open the image on which you want us to add mirrored gallery wrap edges and run this script again.');
	forceQuit = true; // quit
}

if (forceQuit != true) {
	// bring application forward for double-click events
	app.bringToFront();
	
	// Save the current preferences 
	var origRulerUnits = app.preferences.rulerUnits;
	var origDisplayDialogs = app.displayDialogs;
	var initialLayer = activeDocument.activeLayer; //save user's selected layer
	var origBackgroundColor = app.backgroundColor  
	var origForegroundColor = app.foregroundColor 
	app.activeDocument.activeLayer = app.activeDocument.layers[0]; //Set to Top Layer
	
	// Set Adobe Photoshop CS4 to use PIXELS (because best I can tell, all the selection and bounds measurements are based in pixesl no matter what) and display no dialogs 
	app.preferences.rulerUnits = Units.PIXELS;
	app.displayDialogs = DialogModes.NO;
	
	
	var DOCWIDTH = 0; //init global var
	var DOCHEIGHT = 0;//init global var
	var DOCRESOLUTION = app.activeDocument.resolution; //strangely, can't use 'value' for resolution, but required for width & height; sigh.
	
	//get Border Width from User
	var borderWidth=getBorderWidth();
	
	//Created Mirrored Borders by grabbing strips from each of 4 sides, enlarging canvas, copying/pasting a border, fliping them, etc.
	makeGalleryWrap();
	
	//ADD 8 CROP Marks (to help fold canvas when stretching)
	addCropMarks() ; 
	//to prvent confusion, deselect crop mark 
	
	//ADD 4 Guides for Original Canvas
	addGuides();
	
	// restore user settings
	app.preferences.rulerUnits = origRulerUnits;  
	app.displayDialogs = origDisplayDialogs;
	app.backgroundColor  =  origBackgroundColor ;
	app.foregroundColor  =  origForegroundColor ;
	
	//Fit New (enlarged) Image to Screen
	fitToScreen();
	
	//All done
	app.beep();  //Can just barely hear this; CLO To Do: any way to set volume? Specify a # of beeps?
	app.beep();
	app.beep();
	
	alert ("Done! \rBe sure to check your mirrored edges for obviously duplicated unique elements that would disproportionately draw the viewer’s eye (eg, the top of a portrait's head, the resumption of the horizon in the midst of sky, etc.). To remove the distracting items, we\'ll usually clone them out, use the gradient tool (watching out for any resulting \"hard lines\"), or use the border stretching method on just the offending portion. We do any such changes on a non-destructive separate layer (just like you, right?).", "Finito", true)
	
	
	app.bringToFront(); 
}



// get the border width, as number of pixesl
function getBorderWidth() {
	getDocDimensions();

	var msg = 'Gallery Wrap Creator v1.0 - Copyright ©2008 Chris Ogden <InTheMomentImages.com>. \rThis script is released as freeware, as long as this notice remains intact... \r\rThis script will create a new layer with mirrored borders using the width you specify.  It will also add crop marks (to make folding around stretchers easier) to each of the 8 folding points on the canvas. Lastly, it will add Photoshop guides around the original image. \r\rSince separate layers are used, your original image will not be altered, nor will it be saved, allowing you to \"undo\" the results by \"reverting\" to your saved image (assuming you\'ve saved your file before running this script). \r\rYour image is ' + DOCHEIGHT/DOCRESOLUTION + ' inches high by ' + DOCWIDTH/DOCRESOLUTION + ' inches wide. To run the script, please enter your desired border size, in inches, and press \"OK\":';  

	var error = "";
	var valid_answer = false;
	var theAnswer ='2'; //default to 2"	
	
	while (! valid_answer) {
		var theAnswer=prompt(error + msg, theAnswer);
		var border = Number(theAnswer) * DOCRESOLUTION;
		var smallestDim = Math.min(DOCWIDTH, DOCHEIGHT);
		var foo = DOCWIDTH;
		var fee = DOCHEIGHT;
		
		if(theAnswer == null) { //clicked "Cancel"
			return; // quit
		} else if(theAnswer == '') { //click "OK" with nothing typed in
			error = "ERROR: Please enter your desired border width. \r\r";
		} else if (isNaN(border)){
			error = "ERROR: Border must be a valid number. \r\r";				
		} else if (border <= 0) {
			error = "ERROR: Border must be greater than 0. \r\r";
		} else if (border  * 2 >= smallestDim) { //ensure will will have image to wrap -ie, the image's smallest dimension must be more than twice the  border
			error = "ERROR: Border must be less than half of the images' smallest dimension (ie, " + smallestDim +" in). \r\r";
		} else {
			valid_answer = true;
		}
	}
	return border ; //do all in pixels
}

function getDocDimensions() {
	//get current doc dimenaions
	DOCWIDTH = app.activeDocument.width.value;
	DOCHEIGHT = app.activeDocument.height.value;
}

function selectLeftBorder() {
	var topLeft = Array(0,0);
	var topRight = Array(borderWidth, 0);
	var btmLeft = Array(0, DOCHEIGHT);
	var btmRight = Array(borderWidth,DOCHEIGHT);
	var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
	app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
}

function selectRightBorder() {
	var topLeft = Array(DOCWIDTH-borderWidth,0);
	var topRight = Array(DOCWIDTH, 0);
	var btmLeft = Array(DOCWIDTH-borderWidth, DOCHEIGHT);
	var btmRight = Array(DOCWIDTH,DOCHEIGHT);
	var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
	app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
}

function selectTopBorder() {
	var topLeft = Array(0,0);
	var topRight = Array(DOCWIDTH,0);
	var btmLeft = Array(0, borderWidth);
	var btmRight = Array(DOCWIDTH,borderWidth);
	var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
	app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
}
function selectBottomBorder() {
	var topLeft = Array(0,DOCHEIGHT-borderWidth);
	var topRight = Array(DOCWIDTH, DOCHEIGHT-borderWidth);
	var btmLeft = Array(0, DOCHEIGHT);
	var btmRight = Array(DOCWIDTH,DOCHEIGHT);
	var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
	app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
}

function makeGalleryWrap() {
//pg165 for Selection Syntax -- CLO TO - find out how to do seleciton in inches, prob using "UnitType"??
//Many of the properties and methods use the UnitValue type, which combines measurement values with the measurement unit. For information about this type, see the JavaScript Tools Guide. 

//LEFT BORDER
getDocDimensions();
selectLeftBorder();
app.activeDocument.selection.copy(true); //***CAUTION*** This FAILS if no selection. Set "merged"param to true to do a merged copy where all visible layers are copied.
app.activeDocument.flipCanvas(Direction.HORIZONTAL); //NOTE: in CS4, there is NO support for selection based "TRANSFORM", so either have to create new temp doc, flip it, etc. or you have to flip the orig doc canvas, paste, and flip back.
activeDocument.resizeCanvas(DOCWIDTH + borderWidth, DOCHEIGHT, AnchorPosition.MIDDLELEFT); //// Resize the canvas, adding the amount for the new border;  width, height, AnchorPosition  
getDocDimensions();
selectRightBorder(); //Select white space on right (which is really the left; yeah, I know a pain.
app.activeDocument.paste(); 
app.activeDocument.flipCanvas(Direction.HORIZONTAL); //flip back.
activeDocument.activeLayer.name = "LeftBorder";  // name it for debug/later reference  


//RIGHT BORDER
getDocDimensions();
selectRightBorder();
app.activeDocument.selection.copy(true); //***CAUTION*** This FAILS if no selection. Set "merged"param to true to do a merged copy where all visible layers are copied.
app.activeDocument.flipCanvas(Direction.HORIZONTAL); //NOTE: in CS4, there is NO support for selection based "TRANSFORM", so either have to create new temp doc, flip it, etc. or you have to flip the orig doc canvas, paste, and flip back.
activeDocument.resizeCanvas(DOCWIDTH + borderWidth, DOCHEIGHT, AnchorPosition.MIDDLERIGHT); //// Resize the canvas, adding the amount for the new border;  width, height, AnchorPosition  
getDocDimensions();
selectLeftBorder(); //Select white space on right (which is really the left; yeah, I know a pain.
app.activeDocument.paste(); 
app.activeDocument.flipCanvas(Direction.HORIZONTAL); //flip back.
activeDocument.activeLayer.name = "RightBorder";  // name it for debug/later reference  
app.activeDocument.activeLayer.merge();


//TOP BORDER
getDocDimensions();
selectTopBorder();
app.activeDocument.selection.copy(true); //***CAUTION*** This FAILS if no selection. Set "merged"param to true to do a merged copy where all visible layers are copied.
app.activeDocument.flipCanvas(Direction.VERTICAL); //NOTE: in CS4, there is NO support for selection based "TRANSFORM", so either have to create new temp doc, flip it, etc. or you have to flip the orig doc canvas, paste, and flip back.
activeDocument.resizeCanvas(DOCWIDTH, DOCHEIGHT + borderWidth, AnchorPosition.TOPCENTER); //// Resize the canvas, adding the amount for the new border;  width, height, AnchorPosition  
getDocDimensions();
selectBottomBorder(); //Select white space on right (which is really the left; yeah, I know a pain.
app.activeDocument.paste(); 
app.activeDocument.flipCanvas(Direction.VERTICAL); //flip back.
activeDocument.activeLayer.name = "TopBorder";  // name it for debug/later reference  
app.activeDocument.activeLayer.merge();


//BOTTOM BORDER
getDocDimensions();
selectBottomBorder();
app.activeDocument.selection.copy(true); //***CAUTION*** This FAILS if no selection. Set "merged"param to true to do a merged copy where all visible layers are copied.
app.activeDocument.flipCanvas(Direction.VERTICAL); //NOTE: in CS4, there is NO support for selection based "TRANSFORM", so either have to create new temp doc, flip it, etc. or you have to flip the orig doc canvas, paste, and flip back.
activeDocument.resizeCanvas(DOCWIDTH, DOCHEIGHT + borderWidth, AnchorPosition.BOTTOMCENTER); //// Resize the canvas, adding the amount for the new border;  width, height, AnchorPosition  
getDocDimensions();
selectTopBorder(); //Select white space on right (which is really the left; yeah, I know a pain.
app.activeDocument.paste(); 
app.activeDocument.flipCanvas(Direction.VERTICAL); //flip back.
activeDocument.activeLayer.name = "BottomBorder";  // name it for debug/later reference  
app.activeDocument.activeLayer.merge();

activeDocument.activeLayer.name = "GalleryWrapBorders by ChrisOgden";  // name it for debug/later reference  

}

function addCropMarks() {
app.activeDocument.artLayers.add() //put crops on own layer
activeDocument.activeLayer.name = "GalleryWrapBorders CROP/FOLD Marks";  // name it for debug/later reference  

getDocDimensions(); //just to be sure

//var cropBgdWdth = 20; //HALF width of Crop Mark White Background, in pixels
//var cropBgdHgt = 30; //pixels
//var cropLineWdth = 2; //HALF widh of Crop Mark LINE, in pixels

makeVertCropMark(borderWidth,0) ; //TOP, LEFT BORDER
makeVertCropMark(DOCWIDTH-borderWidth,0) ; //TOP, RIGHT BORDER
makeVertCropMark(borderWidth,DOCHEIGHT-22) ; //TOP, LEFT BORDER
makeVertCropMark(DOCWIDTH-borderWidth,DOCHEIGHT-22) ; //TOP, RIGHT BORDER

makeHorizCropMark(0, borderWidth) ; //LEFT, TOP BORDER
makeHorizCropMark(0, DOCHEIGHT-borderWidth) ; //LEFT, BOTTOM BORDER
makeHorizCropMark(DOCWIDTH-22, borderWidth) ; //RIGHT, TOP BORDER
makeHorizCropMark(DOCWIDTH-22, DOCHEIGHT-borderWidth) ; //RIGHT, BOTTOM BORDER

app.activeDocument.selection.deselect(); //deselect so user doesn't get confused

}

function makeVertCropMark(x,y) {
var topLeft = Array(x-20,y); //20pixels on each side
var topRight = Array(x+20, y);
var btmLeft = Array(x-20, y+22);
var btmRight = Array(x+20,y+22);
var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
app.foregroundColor.rgb.hexValue = 'FF00FF'; //set bgnd to white
app.activeDocument.selection.fill(app.foregroundColor); 

var topLeft = Array(x-1,y); //1pixels on each side
var topRight = Array(x+1, y);
var btmLeft = Array(x-1, y+22);
var btmRight = Array(x+1,y+22);
var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
app.foregroundColor.rgb.hexValue = '000000'; //make black crop/fold line
app.activeDocument.selection.fill(app.foregroundColor); 
}

function makeHorizCropMark(x, y) {
var topLeft = Array(x, y-20); //20pixels on each side
var topRight = Array(x+22, y-20);
var btmLeft = Array(x, y+20);
var btmRight = Array(x+22, y+20);
var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
app.foregroundColor.rgb.hexValue = 'FF00FF'; //set bgnd to white
app.activeDocument.selection.fill(app.foregroundColor); 

var topLeft = Array(x, y-1); //1pixels on each side
var topRight = Array(x+22, y-1);
var btmLeft = Array(x, y+1);
var btmRight = Array(x+22, y+1);
var selRegion = Array(topLeft, topRight, btmRight, btmLeft);  //WARNING: it's left, top RIGHT, bottom
app.activeDocument.selection.select(selRegion, SelectionType.REPLACE,0, false); //region, type, feather, antialias
app.foregroundColor.rgb.hexValue = '000000'; //make black crop/fold line
app.activeDocument.selection.fill(app.foregroundColor); 
}

function addGuides() {
	theAnswer  = ""; //for first public release, don't get into complexity of clearing their guides or not
	//theAnswer = confirm("Almost complete - we're about to add Guides to show the original image (wthout the borders). \r\r Before adding the new guides, do you want us to first clear any guides you've already created?",true,'Clear Guides?');
	if (theAnswer == true) {clearGuides(); }
	
	getDocDimensions();
	newGuide(borderWidth,"Vrtc"); //left guide
	newGuide(DOCWIDTH - borderWidth,"Vrtc"); //right guide
	newGuide(borderWidth,"Hrzn"); //top guide
	newGuide(DOCHEIGHT - borderWidth,"Hrzn"); //bottom guide
}

function newGuide(position, type) {  // add "new guide"   types: Vrtc & Hrzn
   // THANKS to http://www.ps-scripts.com/bb/viewtopic.php?t=2244&highlight=guide
   var id296 = charIDToTypeID( "Mk  " );
       var desc50 = new ActionDescriptor();
       var id297 = charIDToTypeID( "Nw  " );
           var desc51 = new ActionDescriptor();
           var id298 = charIDToTypeID( "Pstn" );
           var id299 = charIDToTypeID( "#Pxl" );
           desc51.putUnitDouble( id298, id299, position );
           var id300 = charIDToTypeID( "Ornt" );
           var id301 = charIDToTypeID( "Ornt" );
           var id302 = charIDToTypeID( type );
          desc51.putEnumerated( id300, id301, id302 );
          var id303 = charIDToTypeID( "Gd  " );
       desc50.putObject( id297, id303, desc51 );
   executeAction( id296, desc50, DialogModes.NO );
}; 



function clearGuides() {    // clear all guidelines
   // THANKS to http://www.ps-scripts.com/bb/viewtopic.php?t=2244&highlight=guide
   var id556 = charIDToTypeID( "Dlt " );
   var desc102 = new ActionDescriptor();
   var id557 = charIDToTypeID( "null" );
   var ref70 = new ActionReference();
   var id558 = charIDToTypeID( "Gd  " );
   var id559 = charIDToTypeID( "Ordn" );
   var id560 = charIDToTypeID( "Al  " );
   ref70.putEnumerated( id558, id559, id560 );
    desc102.putReference( id557, ref70 );
   executeAction( id556, desc102, DialogModes.NO );
};

function fitToScreen() { 
	//THANKS to http://www.ps-scripts.com/bb/viewtopic.php?t=1643&highlight=fit+screen
	var id29 = charIDToTypeID( "slct" );
    var desc7 = new ActionDescriptor();
    var id30 = charIDToTypeID( "null" );
        var ref2 = new ActionReference();
        var id31 = charIDToTypeID( "Mn  " );
        var id32 = charIDToTypeID( "MnIt" );
        var id33 = charIDToTypeID( "FtOn" );
        ref2.putEnumerated( id31, id32, id33 );
    desc7.putReference( id30, ref2 );
executeAction( id29, desc7, DialogModes.NO ); 
};

function getAppVersion() {
	// get Photoshop version number
	return parseInt(version);
}



//Thaaaaaaaaaaaaat's all folks (for now). Check out http://InTheMomentImages.com for lots of great photographs, digital darkroom workshops, photo safaris to exotic locales, news about photography and the environment, "How To's" and tutorials, and even more goodies...